/** \file
 *
 *  \brief Video ouput modules & interfaces.
 *
 *  \copyright Copyright 2003-2021 Ciaran Anscomb
 *
 *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
 *
 *  XRoar is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation, either version 3 of the License, or (at your option) any later
 *  version.
 *
 *  See COPYING.GPL for redistribution conditions.
 *
 *  \endlicenseblock
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>

#include "module.h"
#include "vo.h"

extern struct module vo_null_module;
static struct module * const default_vo_module_list[] = {
	&vo_null_module,
	NULL
};

struct module * const *vo_module_list = default_vo_module_list;

struct xconfig_enum vo_cmp_ccr_list[] = {
	{ XC_ENUM_INT("none", VO_CMP_CCR_NONE, "no cross-colour") },
	{ XC_ENUM_INT("simple", VO_CMP_CCR_2BIT, "four colour palette") },
	{ XC_ENUM_INT("5bit", VO_CMP_CCR_5BIT, "5-bit lookup table") },
	{ XC_ENUM_INT("simulated", VO_CMP_CCR_SIMULATED, "simulated filtered analogue") },
};

const uint8_t vo_cmp_lut_2bit[2][4][3] = {
	{
		{ 0x00, 0x00, 0x00 },
		{ 0x00, 0x80, 0xff },
		{ 0xff, 0x80, 0x00 },
		{ 0xff, 0xff, 0xff },
	}, {
		{ 0x00, 0x00, 0x00 },
		{ 0xff, 0x80, 0x00 },
		{ 0x00, 0x80, 0xff },
		{ 0xff, 0xff, 0xff }
	}
};

const uint8_t vo_cmp_lut_5bit[2][32][3] = {
	{
		{ 0x00, 0x00, 0x00 },
		{ 0x00, 0x00, 0x00 },
		{ 0x00, 0x32, 0x78 },
		{ 0x00, 0x28, 0x00 },
		{ 0xff, 0x8c, 0x64 },
		{ 0xff, 0x8c, 0x64 },
		{ 0xff, 0xd2, 0xff },
		{ 0xff, 0xf0, 0xc8 },
		{ 0x00, 0x32, 0x78 },
		{ 0x00, 0x00, 0x3c },
		{ 0x00, 0x80, 0xff },
		{ 0x00, 0x80, 0xff },
		{ 0xd2, 0xff, 0xd2 },
		{ 0xff, 0xff, 0xff },
		{ 0x64, 0xf0, 0xff },
		{ 0xff, 0xff, 0xff },
		{ 0x3c, 0x00, 0x00 },
		{ 0x3c, 0x00, 0x00 },
		{ 0x00, 0x00, 0x00 },
		{ 0x00, 0x28, 0x00 },
		{ 0xff, 0x80, 0x00 },
		{ 0xff, 0x80, 0x00 },
		{ 0xff, 0xff, 0xff },
		{ 0xff, 0xf0, 0xc8 },
		{ 0x28, 0x00, 0x28 },
		{ 0x28, 0x00, 0x28 },
		{ 0x00, 0x80, 0xff },
		{ 0x00, 0x80, 0xff },
		{ 0xff, 0xf0, 0xc8 },
		{ 0xff, 0xf0, 0xc8 },
		{ 0xff, 0xff, 0xff },
		{ 0xff, 0xff, 0xff },
	}, {
		{ 0x00, 0x00, 0x00 },
		{ 0x00, 0x00, 0x00 },
		{ 0xb4, 0x3c, 0x1e },
		{ 0x28, 0x00, 0x28 },
		{ 0x46, 0xc8, 0xff },
		{ 0x46, 0xc8, 0xff },
		{ 0xd2, 0xff, 0xd2 },
		{ 0x64, 0xf0, 0xff },
		{ 0xb4, 0x3c, 0x1e },
		{ 0x3c, 0x00, 0x00 },
		{ 0xff, 0x80, 0x00 },
		{ 0xff, 0x80, 0x00 },
		{ 0xff, 0xd2, 0xff },
		{ 0xff, 0xff, 0xff },
		{ 0xff, 0xf0, 0xc8 },
		{ 0xff, 0xff, 0xff },
		{ 0x00, 0x00, 0x3c },
		{ 0x00, 0x00, 0x3c },
		{ 0x00, 0x00, 0x00 },
		{ 0x28, 0x00, 0x28 },
		{ 0x00, 0x80, 0xff },
		{ 0x00, 0x80, 0xff },
		{ 0xff, 0xff, 0xff },
		{ 0x64, 0xf0, 0xff },
		{ 0x00, 0x28, 0x00 },
		{ 0x00, 0x28, 0x00 },
		{ 0xff, 0x80, 0x00 },
		{ 0xff, 0x80, 0x00 },
		{ 0x64, 0xf0, 0xff },
		{ 0x64, 0xf0, 0xff },
		{ 0xff, 0xff, 0xff },
		{ 0xff, 0xff, 0xff },
	}
};
